/**
 * Created by littwin on 06.01.15.
 */

var app = angular.module('server', ['LocalStorageModule'])
        .config([ 'localStorageServiceProvider', function (localStorageServiceProvider) {
            localStorageServiceProvider.setPrefix("RServerDemo");
        }])
        .factory('User', ['$rootScope', '$location','$q','localStorageService', function ($rootScope, $location,$q, localStorageService) {
            var permissions = 15; // No user system active - allow anything
            var login = '', password = '', session = '';
            var sessionStartedPromise = $q.defer();
            var autologinSend = false;
            service = {
                loggedIn: function () {
                    return permissions != 0;
                },
                realUser: function() {
                    return login != '';
                },
                canPrint: function () {
                    return (permissions & 1) == 1;
                },
                canAddFiles: function () {
                    return (permissions & 2) == 2;
                },
                canDeleteFiles: function () {
                    return (permissions & 4) == 4;
                },
                canChangeConfig: function () {
                    return (permissions & 8) == 8;
                },
                setCredentials: function (_login, _pw) {
                    login = _login;
                    password = _pw;
                },
                logout: function () {
                    permissions = 0;
                    login = "";
                    password = "";
                },
                getSession: function () {
                    return session;
                },
                getSessionEncoded: function() {
                    return encodeURIComponent(session);
                },
                setSession: function (_session) {
                    session = _session;
                    sessionStartedPromise.resolve();
                },
                getLogin: function() {return login;},
                setPermissions: function (perm) {
                    permissions = perm;
                },
                sessionStarted: function() {
                    return sessionStartedPromise.promise;
                },
                autologin: function (RSCom) {
                    console.log('autologin', autologinSend, login, session)
                    if(autologinSend) return;
                    loggedIn = false;
                    autologinSend = true;
                    RSCom.send("login", {
                        login: login,
                        password: CryptoJS.MD5(session + password).toString()
                    }).then(function (data) {
                        if (data.error) {
                            autologinSend = false
                            $rootScope.loginerror = data.error;
                            $rootScope.$broadcast("connectionError",data.error);
                        } else {
                            $rootScope.$broadcast("connected"); // first try was useless due to permission denied
                        }
                    });
                    return true;
                },
                setCredentialsFromServer: function(data) {
                    permissions = data.permissions;
                    login = data.login;
                    localStorageService.add("session",{user:data.login, session:session});
                    $rootScope.$broadcast("connected");
                }
            };
            $rootScope.$on("disconnected",function() {
                autologinSend = false;
            });
            $rootScope.$on('userCredentials',function(event,data) {
                service.setCredentialsFromServer(data.data);
            });

            return service;
        }])
        .factory('RSCom', ['$q', '$rootScope', 'User', '$location', '$http', 'localStorageService', '$timeout',
            function ($q, $rootScope, User, $location, $http, localStorageService, $timeout) {
                $rootScope.connection = {
                    open:false, // should try to open
                    url:'localhost',
                    port:'3344',
                    login:'',
                    password:''
                };

                var Service = {}; // We return this object to anything injecting our service
                var callbacks = {}; // Keep all pending requests here until they get responses
                var currentCallbackId = 0; // Create a unique callback ID to map requests to responses
                var ws; // Create our websocket object with the address to the websocket
                Service.connected = false;
                var deflist = [];
                var printerSlug = '';
                var openPings = 0;
                var firstConnection = true;
                var nextTimeout = 50;
                var startVersion = "";
                var ignoreErrorsTil = -1;
                var startConnection = function () {
                    if($rootScope.connection.open == false) {// loop until we are ready to open
                        $timeout(startConnection, 2000);
                        return;
                    }
                    var auth = localStorageService.get("session");
                    var extra = "";
                    ignoreErrorsTil = -1; // reset error handling and callbacks
                    callbacks = {};
                    currentCallbackId = 0;
                    if (auth && auth.session)
                        extra = "&sess=" + encodeURIComponent(auth.session);
                    ws = new WebSocket("ws://" + $rootScope.connection.url+":"+$rootScope.connection.port + "/socket/?lang=en" + extra);
                    ws.onopen = function () {
                        Service.connected = true;
                        openPings = 0;
                        $.each(deflist, function (idx, val) {
                            val.resolve(ws);
                        });
                        $rootScope.$broadcast("connected");
                        var tryFlush = function () {
                            if(ws.readyState != 1) {
                                $timeout(tryFlush, 20);
                                //$timeout(startConnection,20);
                                return;
                            }
                            firstConnection = false;
                            //log("flush commands");
                            $.each(deflist, function (idx, val) {
                                val.resolve(ws);
                            });
                            deflist = [];
                            //Service.updateImportant();
                            nextTimeout = 50;
                            Service.wasConnected = true;
                        };
                        $timeout(tryFlush, 10);
                        $rootScope.$apply();
                    };
                    ws.onerror = function (message) {
                    };
                    ws.onclose = function (message) {
                        if (Service.connected) {
                            Service.connected = false;
                            $rootScope.$broadcast("disconnected");
                        }
                        setTimeout(startConnection, nextTimeout);
                        if(nextTimeout < 1000)
                            nextTimeout = 2 * nextTimeout;
                        else
                            nextTimeout = 2000;
                    };
                    ws.onmessage = function (message) {
                        try {
                            listener(angular.fromJson(message.data));
                        } catch (error) {
                        }
                    };
                };
                $(window).unload(function () { // force closing connection
                    ws.onclose = function () {
                    };
                    ws.close();
                });
                $rootScope.forceLogin = function () {
                    if (User.autologin(Service)) return;
                    $roorScope.$apply();
                };
                // Server might require login, so we login with known credentials
                $rootScope.$on('loginRequired', function (evt, data) {
                    User.setSession(data.data.session)
                    User.autologin(Service);
                });
                startConnection(); // Start connection loop. Loops forever!

                function sendRequest(request) {
                    // console.log('send', request)
                    var defer = $q.defer();
                    var callbackId = getCallbackId();
                    if(request.action=="login") {
                        ignoreErrorsTil = callbackId;
                    }
                    callbacks[callbackId] = {
                        time: new Date(),
                        cb: defer
                    };
                    request.callback_id = callbackId;
                    if (!Service.connected) {
                        var d = $q.defer();
                        deflist.push(d);
                        d.promise.then(function () {
                            request["session"] = User.getSession();
                            ws.send(angular.toJson(request));
                        }, function () {
                        });
                    } else
                        ws.send(angular.toJson(request));
                    return defer.promise;
                }

                function listener(data) {
                    // console.log('recvt', data)
                    var messageObj = data;
                    if (typeof(messageObj.session) != 'undefined' && User.getSession() != messageObj.session) {
                        User.setSession(messageObj.session);
                    }
                    // If an object exists with callback_id in our callbacks object, resolve it
                    if (messageObj.callback_id < 0) { // event
                        angular.forEach(messageObj.data, function (evt) {
                            $rootScope.$broadcast(evt.event, evt);
                            $rootScope.$broadcast("serverEvent", evt);
                            $rootScope.$apply();
                        });
                    } else if (callbacks[messageObj.callback_id]) {
                        var cb = callbacks[messageObj.callback_id];
                        if(ignoreErrorsTil < 0 && ignoreErrorsTil > messageObj.callback_id)
                            ignoreErrorsTil = -1;
                        if (typeof(messageObj.data.permissionDenied) != 'undefined') {
                            if(ignoreErrorsTil >= 0 && messageObj.callback_id < ignoreErrorsTil) {
                                cb.cb.reject();
                                delete callbacks[messageObj.callback_id];
                            } else {
                                cb.cb.reject();
                                delete callbacks[messageObj.callback_id];
                                $rootScope.forceLogin();
                                $rootScope.$broadcast("PermissionDenied","Permission denied");
                            }
                            return;
                        } else if (typeof(messageObj.data.alert) != 'undefined') {
                            cb.cb.reject();
                            delete callbacks[messageObj.callbackID];
                            $rootScope.alert = messageObj.data.alert;
                            return;
                        }
                        $rootScope.$apply(cb.cb.resolve(messageObj.data));
                        delete callbacks[messageObj.callback_id];
                    }
                }

                // This creates a new callback ID for a request
                function getCallbackId() {
                    currentCallbackId += 1;
                    if (currentCallbackId > 10000) {
                        currentCallbackId = 0;
                    }
                    return currentCallbackId;
                }

                Service.send = function (command, data) {
                    return sendRequest({action: command, data: data, printer: printerSlug});
                };
                Service.selectPrinter = function (slug) {
                    printerSlug = slug;
                };
                Service.close = function() {
                    if(Service.connected  == false) return;
                    $rootScope.connection.open = false;
                    Service.connected = false;
                    ws.close();
                };
                var ping = function () {
                    if (Service.connected) {
                        if (openPings > 10)
                            ws.close();
                        else {
                            openPings++;
                            Service.send("ping", {}).then(function () {
                                openPings--;
                            }); // we are still alive!
                        }
                    }
                    $timeout(ping, 1000);
                };
                ping();
                return Service;
            }]
        )

        .controller('main', ['$scope', '$rootScope','RSCom','User', function ($scope, $rootScope, RSCom, User) {
            $scope.mode = 0;
            $scope.error = false;
            $scope.rscom = RSCom;
            $scope.action = {cmd:'',json:'{}',response:''};
            $scope.events = [];
            $scope.activePrinter = "";
            $scope.updateEvents = true;
            $scope.tryConnect = function() {
                User.setCredentials($rootScope.connection.login, CryptoJS.MD5($rootScope.connection.login + $rootScope.connection.password).toString());
                $rootScope.connection.open = true;
                $scope.mode = 1;
            };
            $scope.disconnect = function() {
                RSCom.close();
                $scope.mode = 0;
                $scope.error = false;
            };
            $rootScope.$on("connectionError",function(evt,msg) {
                $scope.error = msg;
                $scope.mode = 0;
                RSCom.close();
            });

            $rootScope.$on("connected",function() {
                $scope.error = false;
                $scope.updatePrinter(); // get us a fresh list of available printer names
            });

            $rootScope.$on("disconnected",function() {
                $scope.mode = 0;
            });

            $rootScope.$on("serverEvent",function(e,event) {
                if(!$scope.updateEvents)
                    return;
                $scope.events.push(event);
                if($scope.events.length > 10)
                    $scope.events.splice(0,1);
            });
            $scope.updatePrinter = function() {
                RSCom.send("listPrinter",{}).then(function(answer) {
                    $scope.printers = answer;
                    if($scope.activePrinter.length == 0 && answer.length > 0)
                        $scope.activePrinter = answer[0].slug;
                });
            };
            // Send a command and store response
            $scope.sendCommand = function() {
                RSCom.send($scope.action.cmd,angular.fromJson($scope.action.json)).then(function(answer) {
                    $scope.action.response = answer;
                });
            }
        }])
    ;
